@@ -13,7 +13,7 @@ pub(crate) fn execute(
     game: &Game,
     action_rng: &mut ChaCha8Rng,
     _description_rng: &mut ChaCha8Rng,
-) -> Option<ActionOutput> {
+) -> ActionOutput {
     let timer_increase = 6 + action_rng.random_range(0..=6);
 
     let possession = match input.end_at.period() {
@@ -55,7 +55,8 @@ pub(crate) fn execute(
         _ => unreachable!("Invalid period {}", input.end_at.period()),
     };
 
-    let result = ActionOutput {
+    
+    ActionOutput {
         situation: ActionSituation::BallInBackcourt,
         description,
         start_at: input.end_at,
@@ -64,6 +65,5 @@ pub(crate) fn execute(
         home_score: input.home_score,
         away_score: input.away_score,
         ..Default::default()
-    };
-    Some(result)
+    }
 }
