@@ -2,12 +2,13 @@ use super::challenge::Challenge;
 use super::trade::Trade;
 use super::types::{NetworkData, NetworkGame, NetworkRequestState, NetworkTeam, SeedInfo};
 use crate::app_version;
+use crate::core::constants::NETWORK_GAME_START_DELAY;
+use crate::core::{TournamentRegistrationState, MAX_AVG_TIREDNESS_PER_AUTO_GAME};
 use crate::game_engine::types::TeamInGame;
+use crate::game_engine::{Tournament, TournamentId};
 use crate::store::deserialize;
-use crate::types::{AppResult, SystemTimeTick, TeamId, Tick};
-use crate::ui::popup_message::PopupMessage;
-use crate::world::constants::NETWORK_GAME_START_DELAY;
-use crate::world::MAX_AVG_TIREDNESS_PER_AUTO_GAME;
+use crate::types::{AppResult, PlayerMap, SystemTimeTick, TeamId, Tick};
+use crate::ui::PopupMessage;
 use crate::{app::App, types::AppCallback};
 use anyhow::anyhow;
 use libp2p::gossipsub::TopicHash;
@@ -25,8 +26,11 @@ pub enum NetworkCallback {
     },
     PushSwarmPanelLog {
         timestamp: Tick,
+        peer_id: Option<PeerId>,
         text: String,
+        level: log::Level,
     },
+
     BindAddress {
         address: Multiaddr,
     },
@@ -59,8 +63,12 @@ impl NetworkCallback {
 
     fn bind_address(address: Multiaddr) -> AppCallback {
         Box::new(move |app: &mut App| {
-            app.ui
-                .push_log_event(Tick::now(), None, format!("Bound to {address}"));
+            app.ui.push_log_event(
+                Tick::now(),
+                None,
+                format!("Bound to {address}"),
+                log::Level::Debug,
+            );
 
             app.network_handler.dial_seed()?;
 
@@ -70,8 +78,12 @@ impl NetworkCallback {
 
     fn subscribe(topic: TopicHash) -> AppCallback {
         Box::new(move |app: &mut App| {
-            app.ui
-                .push_log_event(Tick::now(), None, format!("Subscribed to topic: {topic}"));
+            app.ui.push_log_event(
+                Tick::now(),
+                None,
+                format!("Subscribed to topic: {topic}"),
+                log::Level::Debug,
+            );
             app.world.dirty_network = true;
             Ok(None)
         })
@@ -83,6 +95,7 @@ impl NetworkCallback {
                 Tick::now(),
                 Some(peer_id),
                 format!("Peer {peer_id} unsubscribed from topic: {topic}"),
+                log::Level::Debug,
             );
             app.world.filter_peer_data(Some(peer_id))?;
             app.ui.swarm_panel.remove_peer_id(&peer_id);
@@ -92,15 +105,15 @@ impl NetworkCallback {
 
     fn close_connection(peer_id: PeerId) -> AppCallback {
         Box::new(move |app: &mut App| {
-            // if !app.network_handler.swarm.is_connected(&peer_id) {
             app.ui.push_log_event(
                 Tick::now(),
                 Some(peer_id),
                 format!("Closing connection: {peer_id}"),
+                log::Level::Debug,
             );
             // FIXME: read connection protocol and understand when this is called.
             //        For example, we could check that num_established >0 or that cause = None
-            // }
+
             Ok(None)
         })
     }
@@ -111,32 +124,35 @@ impl NetworkCallback {
         network_team: NetworkTeam,
     ) -> AppCallback {
         Box::new(move |app: &mut App| {
-            app.ui.push_log_event(
-                Tick::now(),
-                peer_id,
-                format!("Got a team from peer: {peer_id:?}"),
-            );
+            let team_version_updated = app.world.add_network_team(network_team.clone())?;
 
             if let Some(id) = peer_id {
                 app.ui.swarm_panel.add_peer_id(id, network_team.team.id);
             }
-            app.world.add_network_team(network_team.clone())?;
 
-            app.ui.push_log_event(
-                timestamp,
-                peer_id,
-                format!(
-                    "Deserialized team: {} {}",
-                    network_team.team.name, network_team.team.version
-                ),
-            );
+            if team_version_updated {
+                app.ui.push_log_event(
+                    timestamp,
+                    peer_id,
+                    format!(
+                        "Received team {} (version {})",
+                        network_team.team.name, network_team.team.version
+                    ),
+                    log::Level::Info,
+                );
+            }
+
             Ok(None)
         })
     }
 
-    fn handle_message_topic(peer_id: Option<PeerId>, timestamp: Tick, text: String) -> AppCallback {
+    fn handle_message_topic(
+        peer_id: Option<PeerId>,
+        timestamp: Tick,
+        message: String,
+    ) -> AppCallback {
         Box::new(move |app: &mut App| {
-            app.ui.push_chat_event(timestamp, peer_id, text.clone());
+            app.ui.push_chat_event(timestamp, peer_id, message.clone());
             Ok(None)
         })
     }
@@ -159,6 +175,176 @@ impl NetworkCallback {
         })
     }
 
+    fn handle_tournament_registration_request_topic(
+        tournament_id: TournamentId,
+        team_id: TeamId,
+        state: TournamentRegistrationState,
+    ) -> AppCallback {
+        Box::new(move |app: &mut App| {
+            let tournament = if let Some(t) = app.world.tournaments.get_mut(&tournament_id) {
+                t
+            } else {
+                return Err(anyhow!("No tournament with id {tournament_id} found."));
+            };
+
+            // Check that state and tournament_id are consistent
+            match state {
+                TournamentRegistrationState::Pending {
+                    tournament_id: state_tid,
+                }
+                | TournamentRegistrationState::Confirmed {
+                    tournament_id: state_tid,
+                } => {
+                    if tournament_id != state_tid {
+                        return Err(anyhow!(
+                            "Inconsistent tournament ids: {tournament_id} and {state_tid}"
+                        ));
+                    }
+                }
+                TournamentRegistrationState::None => {}
+            }
+
+            // Check if own_team is tournament organizer.
+            // In this case, we register the team to the tournament and send back the response.
+            if tournament.organizer_id == app.world.own_team_id {
+                // We process only requests with state = TournamentRegistrationState::Pending { tournament_id }
+                if !matches!(state, TournamentRegistrationState::Pending { .. }) {
+                    app.network_handler.send_tournament_registration_request(
+                        tournament_id,
+                        team_id,
+                        TournamentRegistrationState::None,
+                    )?;
+                    app.ui.push_log_event(
+                        Tick::now(),
+                        None,
+                        format!(
+                            "Registration declined by organizer: Invalid tournament state received: {state}"
+                        ),
+                        log::Level::Error,
+                    );
+                }
+
+                let team = app
+                    .world
+                    .teams
+                    .get(&team_id)
+                    .ok_or(anyhow!("Team {team_id} not found."))?;
+                let mut players = PlayerMap::new();
+                for player_id in team.player_ids.iter() {
+                    let player = app
+                        .world
+                        .players
+                        .get(player_id)
+                        .ok_or(anyhow!("Player {player_id} not found."))?
+                        .clone();
+                    players.insert(player.id, player);
+                }
+                match tournament.register_team(team, players) {
+                    Ok(()) => {
+                        app.network_handler.send_tournament_registration_request(
+                            tournament_id,
+                            team_id,
+                            TournamentRegistrationState::Confirmed { tournament_id },
+                        )?;
+                        app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} confirmed for team {team_id}."), log::Level::Info);
+                        app.network_handler.send_tournament(tournament.clone())?;
+                    }
+                    Err(err) => {
+                        app.network_handler.send_tournament_registration_request(
+                            tournament_id,
+                            team_id,
+                            TournamentRegistrationState::None,
+                        )?;
+                        app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} declined for team {team_id}: {err}"), log::Level::Warn);
+                    }
+                }
+            }
+            // Check if own_team is team_id (sender of request).
+            // In this case we intepret the state as a confirmation or a cancelation,
+            // if the state of the team is pending and the tournament id matches.
+            else if team_id == app.world.own_team_id {
+                let own_team = app.world.get_own_team_mut()?;
+                if !matches!(
+                    own_team.tournament_registration_state,
+                    TournamentRegistrationState::Pending {tournament_id: id} if id == tournament_id
+                ) {
+                    // Discard message
+                    app.ui.push_log_event(
+                        Tick::now(),
+                        None,
+                        format!("Invalid tournament state received: {state}"),
+                        log::Level::Error,
+                    );
+                }
+
+                // We process only requests with state = TournamentRegistrationState::Confirmed { tournament_id } or None
+                match state {
+                    TournamentRegistrationState::None => {
+                        app.ui.push_log_event(
+                            Tick::now(),
+                            None,
+                            format!("Registration to tournament {tournament_id} declined."),
+                            log::Level::Warn,
+                        );
+                        own_team.tournament_registration_state = TournamentRegistrationState::None;
+                    }
+                    TournamentRegistrationState::Pending { tournament_id } => {
+                        app.ui.push_log_event(
+                            Tick::now(),
+                            None,
+                            format!("Invalid tournament {tournament_id} state received: {state}"),
+                            log::Level::Error,
+                        );
+                    }
+                    TournamentRegistrationState::Confirmed { tournament_id } => {
+                        app.ui.push_log_event(
+                            Tick::now(),
+                            None,
+                            format!("Registration to tournament {tournament_id} accepted."),
+                            log::Level::Info,
+                        );
+                        own_team.tournament_registration_state =
+                            TournamentRegistrationState::Confirmed { tournament_id };
+                    }
+                }
+            }
+
+            Ok(None)
+        })
+    }
+
+    fn handle_tournament_topic(tournament: Tournament) -> AppCallback {
+        Box::new(move |app: &mut App| {
+            if tournament.organizer_id == app.world.own_team_id {
+                return Err(anyhow!(
+                    "Cannot receive tournament organized by oneself over the network."
+                ));
+            }
+
+            if !tournament.has_started(Tick::now())
+                && tournament.has_available_spots()
+                && !app.world.tournaments.contains_key(&tournament.id)
+            {
+                let planet = app.world.get_planet_or_err(&tournament.planet_id)?;
+                app.ui.push_popup_to_top(PopupMessage::Ok {
+                    message: format!(
+                        "There is a tournament on {} starting in {} with available spots.",
+                        planet.name,
+                        (tournament.starting_at - Tick::now()).formatted_as_time()
+                    ),
+                    is_skippable: true,
+                    tick: Tick::now(),
+                });
+            }
+
+            app.world
+                .tournaments
+                .insert(tournament.id, tournament.clone());
+
+            Ok(None)
+        })
+    }
+
     fn handle_game_topic(
         peer_id: Option<PeerId>,
         timestamp: Tick,
@@ -168,7 +354,11 @@ impl NetworkCallback {
             app.ui.push_log_event(
                 timestamp,
                 peer_id,
-                format!("Deserialized game {} from peer {:?}", game.id, peer_id),
+                format!(
+                    "Deserialized game {}:  {} vs {}",
+                    game.id, game.home_team_in_game.name, game.away_team_in_game.name
+                ),
+                log::Level::Info,
             );
             app.world.add_network_game(game.clone())?;
             Ok(None)
@@ -186,6 +376,7 @@ impl NetworkCallback {
                 timestamp,
                 peer_id,
                 format!("Total peers: {}", seed_info.connected_peers_count),
+                log::Level::Info,
             );
 
             if let Some(message) = seed_info.message.clone() {
@@ -217,11 +408,8 @@ impl NetworkCallback {
             app.ui.push_log_event(
                 timestamp,
                 peer_id,
-                format!(
-                    "Deserialized trade from peer {:?}: {}",
-                    peer_id,
-                    trade.format()
-                ),
+                format!("Deserialized trade: {}", trade.format()),
+                log::Level::Info,
             );
 
             let self_peer_id = app.network_handler.own_peer_id();
@@ -293,6 +481,7 @@ impl NetworkCallback {
                             timestamp,
                             peer_id,
                             "Trade accepted, players swapped".to_string(),
+                            log::Level::Info,
                         );
 
                         app.ui.push_popup(PopupMessage::Ok {
@@ -326,6 +515,7 @@ impl NetworkCallback {
                             timestamp,
                             peer_id,
                             "A trade is happening in the network".to_string(),
+                            log::Level::Info,
                         );
 
                         return Ok(None);
@@ -378,6 +568,7 @@ impl NetworkCallback {
                             timestamp,
                             peer_id,
                             "Trade accepted, players swapped".to_string(),
+                            log::Level::Info,
                         );
 
                         app.ui.push_popup(PopupMessage::Ok {
@@ -430,7 +621,8 @@ impl NetworkCallback {
             app.ui.push_log_event(
                 timestamp,
                 peer_id,
-                format!("\nChallenge: {}", challenge.format()),
+                format!("Challenge: {}", challenge.format()),
+                log::Level::Info,
             );
 
             let self_peer_id = app.network_handler.own_peer_id();
@@ -461,6 +653,7 @@ impl NetworkCallback {
                     let own_team = app.world.get_own_team_mut()?;
 
                     if own_team.current_game.is_none()
+                        && own_team.committed_to_tournament().is_none()
                         && own_team.autonomous_strategy.challenge_network
                         && average_tiredness <= MAX_AVG_TIREDNESS_PER_AUTO_GAME
                     {
@@ -519,6 +712,7 @@ impl NetworkCallback {
                             timestamp,
                             peer_id,
                             "Challenge accepted, generating game".to_string(),
+                            log::Level::Info,
                         );
 
                         let own_team = app.world.get_own_team_mut()?;
@@ -570,6 +764,7 @@ impl NetworkCallback {
                             timestamp,
                             peer_id,
                             "Adding challenge from network".to_string(),
+                            log::Level::Info,
                         );
 
                         if let Some(starting_at) = challenge.starting_at {
@@ -598,6 +793,7 @@ impl NetworkCallback {
                             timestamp,
                             peer_id,
                             "Challenge accepted, generating game".to_string(),
+                            log::Level::Info,
                         );
 
                         if let Some(starting_at) = challenge.starting_at {
@@ -672,8 +868,14 @@ impl NetworkCallback {
                 peer_id,
                 text,
             } => Self::push_swarm_panel_message(*timestamp, *peer_id, text.clone())(app),
-            Self::PushSwarmPanelLog { timestamp, text } => {
-                app.ui.push_log_event(*timestamp, None, text.clone());
+            Self::PushSwarmPanelLog {
+                timestamp,
+                peer_id,
+                text,
+                level,
+            } => {
+                app.ui
+                    .push_log_event(*timestamp, *peer_id, text.clone(), *level);
                 Ok(None)
             }
             Self::BindAddress { address } => Self::bind_address(address.clone())(app),
@@ -687,6 +889,7 @@ impl NetworkCallback {
                     Tick::now(),
                     Some(*peer_id),
                     format!("Connected to peer: {peer_id}"),
+                    log::Level::Debug,
                 );
                 Ok(None)
             }
@@ -695,27 +898,46 @@ impl NetworkCallback {
 
                 let network_data = deserialize::<NetworkData>(&message.data)?;
                 match network_data {
-                    NetworkData::Team(timestamp, team) => {
+                    NetworkData::Team { timestamp, team } => {
                         Self::handle_team_topic(peer_id, timestamp, team)(app)
                     }
-                    NetworkData::Message(timestamp, text) => {
-                        Self::handle_message_topic(peer_id, timestamp, text)(app)
+                    NetworkData::Message { timestamp, message } => {
+                        Self::handle_message_topic(peer_id, timestamp, message)(app)
                     }
-                    NetworkData::Challenge(timestamp, challenge) => {
-                        Self::handle_challenge_topic(peer_id, timestamp, challenge)(app)
-                    }
-                    NetworkData::Trade(timestamp, trade) => {
+                    NetworkData::Challenge {
+                        timestamp,
+                        challenge,
+                    } => Self::handle_challenge_topic(peer_id, timestamp, challenge)(app),
+                    NetworkData::Trade { timestamp, trade } => {
                         Self::handle_trade_topic(peer_id, timestamp, trade)(app)
                     }
-                    NetworkData::Game(timestamp, game) => {
+                    NetworkData::Game { timestamp, game } => {
                         Self::handle_game_topic(peer_id, timestamp, game)(app)
                     }
-                    NetworkData::SeedInfo(timestamp, seed_info) => {
-                        Self::handle_seed_topic(peer_id, timestamp, seed_info)(app)
-                    }
-                    NetworkData::RelayerMessageToTeam(timestamp, message, team_id) => {
+                    NetworkData::SeedInfo {
+                        timestamp,
+                        seed_info,
+                    } => Self::handle_seed_topic(peer_id, timestamp, seed_info)(app),
+                    NetworkData::RelayerMessageToTeam {
+                        timestamp,
+                        message,
+                        team_id,
+                    } => {
                         Self::handle_relayer_message_to_team_topic(timestamp, message, team_id)(app)
                     }
+                    NetworkData::TournamentRegistrationRequest {
+                        tournament_id,
+                        team_id,
+                        state,
+                        ..
+                    } => Self::handle_tournament_registration_request_topic(
+                        tournament_id,
+                        team_id,
+                        state,
+                    )(app),
+                    NetworkData::Tournament { tournament, .. } => {
+                        Self::handle_tournament_topic(tournament)(app)
+                    }
                 }
             }
         }
