@@ -1,15 +1,17 @@
 use super::challenge::Challenge;
 use super::trade::Trade;
+use crate::core::planet::Planet;
+use crate::core::position::{GamePosition, MAX_GAME_POSITION};
+use crate::core::skill::Skill;
+use crate::core::TournamentRegistrationState;
 use crate::game_engine::timer::Timer;
 use crate::game_engine::types::GameStats;
+use crate::game_engine::{Tournament, TournamentId};
 use crate::types::{PlanetId, PlayerId, Tick};
-use crate::world::planet::Planet;
-use crate::world::position::{Position, MAX_POSITION};
-use crate::world::skill::Skill;
 use crate::{
+    core::{player::Player, team::Team, world::World},
     game_engine::types::TeamInGame,
     types::{AppResult, GameId, TeamId},
-    world::{player::Player, team::Team, world::World},
 };
 use anyhow::anyhow;
 use itertools::Itertools;
@@ -20,15 +22,47 @@ use std::fmt::Debug;
 use strum_macros::Display;
 
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
-#[repr(u8)]
+#[allow(clippy::large_enum_variant)]
 pub enum NetworkData {
-    Team(Tick, NetworkTeam),
-    Challenge(Tick, Challenge),
-    Trade(Tick, Trade),
-    Message(Tick, String),
-    Game(Tick, NetworkGame),
-    SeedInfo(Tick, SeedInfo),
-    RelayerMessageToTeam(Tick, String, TeamId),
+    Team {
+        timestamp: Tick,
+        team: NetworkTeam,
+    },
+    Challenge {
+        timestamp: Tick,
+        challenge: Challenge,
+    },
+    Trade {
+        timestamp: Tick,
+        trade: Trade,
+    },
+    Message {
+        timestamp: Tick,
+        message: String,
+    },
+    Game {
+        timestamp: Tick,
+        game: NetworkGame,
+    },
+    SeedInfo {
+        timestamp: Tick,
+        seed_info: SeedInfo,
+    },
+    RelayerMessageToTeam {
+        timestamp: Tick,
+        message: String,
+        team_id: TeamId,
+    },
+    TournamentRegistrationRequest {
+        timestamp: Tick,
+        tournament_id: TournamentId,
+        team_id: TeamId,
+        state: TournamentRegistrationState,
+    },
+    Tournament {
+        timestamp: Tick,
+        tournament: Tournament,
+    },
 }
 
 #[derive(Debug, Clone, Display, Default, Serialize, Deserialize, PartialEq, Hash)]
@@ -94,6 +128,7 @@ pub struct NetworkGame {
     pub attendance: u32,
     pub starting_at: Tick,
     pub timer: Timer,
+    pub part_of_tournament: Option<TournamentId>,
 }
 
 impl NetworkGame {
@@ -106,8 +141,8 @@ impl NetworkGame {
         for (idx, player_id) in home_team_in_game.initial_positions.iter().enumerate() {
             // Set position in stats to initial one
             let mut player_stats = GameStats::default();
-            if (idx as Position) < MAX_POSITION {
-                player_stats.position = Some(idx as Position);
+            if (idx as GamePosition) < MAX_GAME_POSITION {
+                player_stats.position = Some(idx as GamePosition);
             }
             stats.insert(*player_id, player_stats.clone());
 
@@ -127,8 +162,8 @@ impl NetworkGame {
         let mut stats = HashMap::new();
         for (idx, player_id) in away_team_in_game.initial_positions.iter().enumerate() {
             let mut player_stats = GameStats::default();
-            if (idx as Position) < MAX_POSITION {
-                player_stats.position = Some(idx as Position);
+            if (idx as GamePosition) < MAX_GAME_POSITION {
+                player_stats.position = Some(idx as GamePosition);
             }
             stats.insert(*player_id, player_stats.clone());
 
@@ -152,6 +187,7 @@ impl NetworkGame {
             attendance: game.attendance,
             starting_at: game.starting_at,
             timer: game.timer,
+            part_of_tournament: game.part_of_tournament,
         })
     }
 }
