@@ -1,12 +1,13 @@
 use crate::{
     app::App,
+    core::{
+        constants::*, kartoffel::Kartoffel, planet::Planet, player::Player, resources::Resource,
+        team::Team,
+    },
     game_engine::{
         game::{Game, GameSummary},
         types::GameStatsMap,
-    },
-    world::{
-        constants::*, kartoffel::Kartoffel, planet::Planet, player::Player, resources::Resource,
-        team::Team,
+        Tournament, TournamentId,
     },
 };
 use anyhow::anyhow;
@@ -34,6 +35,7 @@ pub type PlanetMap = HashMap<PlanetId, Planet>;
 pub type GameMap = HashMap<GameId, Game>;
 pub type GameSummaryMap = HashMap<GameId, GameSummary>;
 pub type KartoffelMap = HashMap<KartoffelId, Kartoffel>;
+pub type TournamentMap = HashMap<TournamentId, Tournament>;
 pub type ResourceMap = HashMap<Resource, u32>;
 
 pub trait StorableResourceMap {
@@ -255,8 +257,8 @@ impl SystemTimeTick for Tick {
 mod tests {
     use super::{AppResult, ResourceMap, StorableResourceMap};
     use crate::{
+        core::{resources::Resource, DAYS},
         types::{SystemTimeTick, Tick, SECONDS},
-        world::{resources::Resource, DAYS},
     };
     use rand::{Rng, SeedableRng};
     use rand_chacha::ChaCha8Rng;
