@@ -478,7 +478,7 @@ impl InteractiveStatefulWidget for ClickableTable<'_> {
         };
 
         let is_hovered = callback_registry.is_hovering(area)
-            && callback_registry.get_max_layer() == self.layer();
+            && callback_registry.get_active_layer() == self.layer();
 
         if is_hovered {
             callback_registry.register_mouse_callback(
