@@ -7,11 +7,12 @@ use super::ui_frame::UiFrame;
 use super::widgets::{space_adventure_button, thick_block};
 use super::{traits::Screen, widgets::default_block};
 use crate::types::{AppResult, PlayerId, SystemTimeTick, TeamId};
+use crate::ui::traits::UiStyled;
 use crate::ui::utils::format_au;
 use crate::ui::{constants::*, ui_key};
 use crate::{
+    core::*,
     types::{PlanetId, PlanetMap},
-    world::*,
 };
 use core::fmt::Debug;
 use crossterm::event::{KeyCode, KeyEvent};
@@ -453,7 +454,11 @@ impl GalaxyPanel {
 
                 player_planet_id == team_planet_id
             })
-            .sorted_by(|a, b| b.rating().cmp(&a.rating()))
+            .sorted_by(|a, b| {
+                b.rating()
+                    .partial_cmp(&a.rating())
+                    .expect("Skill should exist")
+            })
             .map(|player| {
                 let name_length = 2 * MAX_NAME_LENGTH + 2;
                 let text = format!(
@@ -472,7 +477,7 @@ impl GalaxyPanel {
             .sorted_by(|a, b| b.1.cmp(a.1))
             .map(|(resource, &amount)| {
                 let text = format!("{:<7} {}", resource.to_string(), (amount as f32).stars(),);
-                (text, UiStyle::DEFAULT)
+                (text, resource.style())
             })
             .collect::<Vec<(String, Style)>>();
 
@@ -495,7 +500,7 @@ impl GalaxyPanel {
         };
 
         let split = Layout::vertical([
-            Constraint::Length(15),
+            Constraint::Length(18), // = Max depth = 5 + Teleport/travel/adventure button
             Constraint::Length(team_list_height),
             Constraint::Length(player_list_height),
             Constraint::Length(resource_list_height),
