@@ -11,19 +11,23 @@ mod hover_text_span;
 mod my_team_panel;
 mod new_team_screen;
 mod player_panel;
-pub mod popup_message;
+mod popup_message;
 mod space_cove_panel;
 mod space_screen;
 mod splash_screen;
 mod swarm_panel;
 mod team_panel;
-pub(crate) mod traits;
-pub mod ui;
-pub mod ui_callback;
+mod tournament_panel;
+mod traits;
+mod ui_callback;
 mod ui_frame;
-pub mod ui_key;
-pub mod utils;
-pub mod widgets;
+mod ui_key;
+mod ui_screen;
+mod utils;
+mod widgets;
 
 pub use constants::UI_SCREEN_SIZE;
+pub use popup_message::*;
+pub use ui_callback::UiCallback;
 pub use ui_key::*;
+pub use ui_screen::{UiScreen, UiState};
