@@ -12,20 +12,20 @@ use super::{
     utils::img_to_lines,
     widgets::{default_block, selectable_list},
 };
+use crate::core::constants::MIN_PLAYERS_PER_GAME;
+use crate::core::team::Team;
 use crate::image::spaceship::{SPACESHIP_IMAGE_HEIGHT, SPACESHIP_IMAGE_WIDTH};
 use crate::types::AppResult;
 use crate::ui::ui_key;
-use crate::world::constants::MIN_PLAYERS_PER_GAME;
-use crate::world::team::Team;
 use crate::{
-    image::game::floor_from_size,
-    image::player::{PLAYER_IMAGE_HEIGHT, PLAYER_IMAGE_WIDTH},
-    types::{PlayerId, TeamId},
-    world::{
-        position::{GamePosition, Position},
+    core::{
+        position::{GamePosition, GamePositionUtils},
         skill::Rated,
         world::World,
     },
+    image::game::floor_from_size,
+    image::player::{PLAYER_IMAGE_HEIGHT, PLAYER_IMAGE_WIDTH},
+    types::{PlayerId, TeamId},
 };
 use core::fmt::Debug;
 use crossterm::event::KeyCode;
@@ -51,20 +51,20 @@ pub enum TeamView {
 impl TeamView {
     fn next(&self) -> Self {
         match self {
-            TeamView::All => TeamView::OpenToChallenge,
-            TeamView::OpenToChallenge => TeamView::Peers,
-            TeamView::Peers => TeamView::All,
+            Self::All => TeamView::OpenToChallenge,
+            Self::OpenToChallenge => TeamView::Peers,
+            Self::Peers => TeamView::All,
         }
     }
 
     fn rule(&self, team: &Team, own_team: &Team) -> bool {
         match self {
-            TeamView::All => true,
-            TeamView::OpenToChallenge => {
+            Self::All => true,
+            Self::OpenToChallenge => {
                 own_team.can_challenge_local_team(team).is_ok()
                     || own_team.can_challenge_network_team(team).is_ok()
             }
-            TeamView::Peers => team.peer_id.is_some(),
+            Self::Peers => team.peer_id.is_some(),
         }
     }
 }
@@ -121,7 +121,7 @@ impl TeamListPanel {
             Constraint::Length(3),
             Constraint::Length(3),
             Constraint::Length(3),
-            Constraint::Min(1),
+            Constraint::Fill(1),
         ])
         .split(area);
 
@@ -219,7 +219,7 @@ impl TeamListPanel {
             Constraint::Length(1),                              //floor
             Constraint::Length(1),                              //name
             Constraint::Length(2),                              //rating
-            Constraint::Min(1),                                 //bottom
+            Constraint::Fill(1),                                //bottom
         ])
         .split(area);
 
@@ -232,20 +232,14 @@ impl TeamListPanel {
             }),
         );
 
-        let side_length = if area.width > (PLAYER_IMAGE_WIDTH as u16 + 2) * 5 {
-            (area.width - (PLAYER_IMAGE_WIDTH as u16 + 2) * 5) / 2
-        } else {
-            0
-        };
-
         let constraints = [
-            Constraint::Min(side_length),
+            Constraint::Fill(1),
             Constraint::Min(PLAYER_IMAGE_WIDTH as u16 + 2),
             Constraint::Min(PLAYER_IMAGE_WIDTH as u16 + 2),
             Constraint::Min(PLAYER_IMAGE_WIDTH as u16 + 2),
             Constraint::Min(PLAYER_IMAGE_WIDTH as u16 + 2),
             Constraint::Min(PLAYER_IMAGE_WIDTH as u16 + 2),
-            Constraint::Min(side_length),
+            Constraint::Fill(1),
         ];
 
         let player_img_split = Layout::horizontal(constraints).split(vertical_split[0]);
@@ -293,8 +287,8 @@ impl TeamListPanel {
             frame.render_widget(
                 Paragraph::new(format!(
                     "{} {}",
-                    (i as Position).as_str(),
-                    (i as Position)
+                    (i as GamePosition).as_str(),
+                    (i as GamePosition)
                         .player_rating(player.current_skill_array())
                         .stars()
                 ))
@@ -310,7 +304,7 @@ impl TeamListPanel {
         .split(vertical_split[4]);
 
         let bench_out_split =
-            Layout::vertical([Constraint::Length(6), Constraint::Min(0)]).split(bottom_split[0]);
+            Layout::vertical([Constraint::Length(6), Constraint::Fill(1)]).split(bottom_split[0]);
 
         frame.render_widget(default_block().title("Bench"), bench_out_split[0]);
         frame.render_widget(default_block().title("Out"), bench_out_split[1]);
@@ -319,7 +313,7 @@ impl TeamListPanel {
             let out_row_split = Layout::vertical([
                 Constraint::Length(4),
                 Constraint::Length(4),
-                Constraint::Min(0),
+                Constraint::Fill(1),
             ])
             .split(bench_out_split[1].inner(Margin {
                 horizontal: 2,
@@ -348,7 +342,7 @@ impl TeamListPanel {
                 if let Some(player) = world.get_player(player_id) {
                     let info = format!("{}\n", player.info.short_name());
                     let skills = player.current_skill_array();
-                    let best_role = Position::best(skills);
+                    let best_role = GamePosition::best(skills);
 
                     let role_info = format!(
                         "{:<2} {:<5}",
@@ -414,13 +408,18 @@ impl TeamListPanel {
 
         let box_split = Layout::vertical([
             Constraint::Length(PLAYER_IMAGE_HEIGHT as u16 / 2 + 4),
-            Constraint::Min(0),
+            Constraint::Fill(1),
         ])
         .split(area);
 
         frame.render_widget(
             default_block()
-                .title(format!(" {} ", team.name))
+                .title(format!(
+                    "{} {}/{}",
+                    team.name,
+                    ui_key::NEXT_SELECTION,
+                    ui_key::PREVIOUS_SELECTION
+                ))
                 .title_alignment(Alignment::Left),
             box_split[0],
         );
@@ -444,24 +443,23 @@ impl Screen for TeamListPanel {
         if world.dirty_ui || self.all_team_ids.len() != world.teams.len() {
             self.all_team_ids = world.teams.keys().copied().collect();
             self.all_team_ids.sort_by(|a, b| {
-                let a = world.get_team_or_err(a).unwrap();
-                let b = world.get_team_or_err(b).unwrap();
                 world
-                    .team_rating(&b.id)
+                    .team_rating(b)
                     .unwrap_or_default()
-                    .partial_cmp(&world.team_rating(&a.id).unwrap_or_default())
+                    .partial_cmp(&world.team_rating(a).unwrap_or_default())
                     .expect("Rating should exists")
             });
             self.update_view = true;
         }
 
         if self.update_view {
+            let own_team = world.get_own_team()?;
             self.team_ids = self
                 .all_team_ids
                 .iter()
                 .filter(|&team_id| {
                     let team = world.get_team_or_err(team_id).unwrap();
-                    self.view.rule(team, world.get_own_team().unwrap())
+                    self.view.rule(team, own_team)
                 })
                 .copied()
                 .collect();
@@ -499,17 +497,6 @@ impl Screen for TeamListPanel {
 
         _debug_view: bool,
     ) -> AppResult<()> {
-        if self.all_team_ids.is_empty() {
-            frame.render_widget(
-                Paragraph::new(" No team yet!"),
-                area.inner(Margin {
-                    vertical: 1,
-                    horizontal: 1,
-                }),
-            );
-            return Ok(());
-        }
-
         // Split into left and right panels
         let left_right_split = Layout::horizontal([
             Constraint::Length(LEFT_PANEL_WIDTH),
@@ -517,6 +504,13 @@ impl Screen for TeamListPanel {
         ])
         .split(area);
         self.build_left_panel(frame, world, left_right_split[0]);
+        if self.all_team_ids.is_empty() {
+            frame.render_widget(
+                Paragraph::new(" No teams yet!"),
+                left_right_split[1].inner(Margin::new(1, 1)),
+            );
+            return Ok(());
+        }
         self.build_right_panel(frame, world, left_right_split[1])?;
         Ok(())
     }
