@@ -7,14 +7,19 @@ use super::{
     popup_message::PopupMessage,
     team_panel::TeamView,
     traits::{Screen, SplitPanel},
-    ui::{UiState, UiTab},
+    ui_screen::{UiState, UiTab},
 };
 use crate::app_version;
+use crate::core::{AsteroidUpgradeTarget, UpgradeableElement};
+use crate::game_engine::game::Game;
+use crate::game_engine::Tournament;
 use crate::network::{challenge::Challenge, trade::Trade};
+use crate::types::PlayerMap;
+use crate::ui::tournament_panel::TournamentView;
 use crate::ui::ui_key;
-use crate::world::{AsteroidUpgradeTarget, UpgradeableElement};
 use crate::{
     app::App,
+    core::*,
     game_engine::{tactic::Tactic, types::TeamInGame},
     image::color_map::{ColorMap, ColorPreset},
     space_adventure::{ControllableSpaceship, PlayerInput, SpaceAdventure},
@@ -22,7 +27,6 @@ use crate::{
         AppCallback, AppResult, GameId, PlanetId, PlayerId, StorableResourceMap, SystemTimeTick,
         TeamId, Tick,
     },
-    world::*,
 };
 use anyhow::anyhow;
 use crossterm::event::{KeyCode, MouseEvent, MouseEventKind};
@@ -55,6 +59,9 @@ pub enum UiCallback {
     GoToGame {
         game_id: GameId,
     },
+    GoToLoadedGame {
+        game: Game,
+    },
     GoToPlanet {
         planet_id: PlanetId,
     },
@@ -144,6 +151,9 @@ pub enum UiCallback {
     SetTeamPanelView {
         view: TeamView,
     },
+    SetTournamentPanelView {
+        view: TournamentView,
+    },
     AbandonAsteroid {
         asteroid_id: PlanetId,
     },
@@ -166,6 +176,11 @@ pub enum UiCallback {
     Drink {
         player_id: PlayerId,
     },
+
+    OrganizeNewTournament {
+        max_participants: usize,
+        starting_at: Tick,
+    },
     GeneratePlayerTeam {
         name: String,
         home_planet: PlanetId,
@@ -245,7 +260,7 @@ impl UiCallback {
             {
                 app.ui.team_panel.set_index(index);
                 app.ui.team_panel.player_index = 0;
-                app.ui.switch_to(super::ui::UiTab::Crews);
+                app.ui.switch_to(super::ui_screen::UiTab::Crews);
             }
             Ok(None)
         })
@@ -253,7 +268,6 @@ impl UiCallback {
 
     fn go_to_player(player_id: PlayerId) -> AppCallback {
         Box::new(move |app: &mut App| {
-            app.ui.player_panel.update(&app.world)?;
             app.ui.player_panel.reset_view();
             if let Some(index) = app
                 .ui
@@ -263,7 +277,9 @@ impl UiCallback {
                 .position(|&x| x == player_id)
             {
                 app.ui.player_panel.set_index(index);
-                app.ui.switch_to(super::ui::UiTab::Pirates);
+                // Update here to update the selected_player_id
+                app.ui.player_panel.update(&app.world)?;
+                app.ui.switch_to(super::ui_screen::UiTab::Pirates);
             }
 
             Ok(None)
@@ -295,7 +311,7 @@ impl UiCallback {
 
                 app.ui.player_panel.locked_player_id = Some(locked_player_id);
                 app.ui.player_panel.selected_player_id = selected_player_id;
-                app.ui.switch_to(super::ui::UiTab::Pirates);
+                app.ui.switch_to(super::ui_screen::UiTab::Pirates);
             }
 
             Ok(None)
@@ -329,7 +345,7 @@ impl UiCallback {
                     .position(|&x| x == player_id)
                     .unwrap_or_default();
                 app.ui.team_panel.player_index = player_index;
-                app.ui.switch_to(super::ui::UiTab::Crews);
+                app.ui.switch_to(super::ui_screen::UiTab::Crews);
             }
 
             Ok(None)
@@ -340,7 +356,17 @@ impl UiCallback {
         Box::new(move |app: &mut App| {
             app.ui.game_panel.update(&app.world)?;
             app.ui.game_panel.set_active_game(game_id)?;
-            app.ui.switch_to(super::ui::UiTab::Games);
+            app.ui.switch_to(super::ui_screen::UiTab::Games);
+            Ok(None)
+        })
+    }
+
+    fn go_to_loaded_game(game: Game) -> AppCallback {
+        Box::new(move |app: &mut App| {
+            app.ui.game_panel.update(&app.world)?;
+            app.ui.game_panel.add_load_game(game.clone());
+            app.ui.game_panel.set_active_game(game.id)?;
+            app.ui.switch_to(super::ui_screen::UiTab::Games);
             Ok(None)
         })
     }
@@ -349,7 +375,7 @@ impl UiCallback {
         Box::new(move |app: &mut App| {
             app.ui.galaxy_panel.update(&app.world)?;
             app.ui.galaxy_panel.go_to_planet(planet_id, ZoomLevel::In);
-            app.ui.switch_to(super::ui::UiTab::Galaxy);
+            app.ui.switch_to(super::ui_screen::UiTab::Galaxy);
 
             Ok(None)
         })
@@ -358,7 +384,7 @@ impl UiCallback {
     fn go_to_space_cove() -> AppCallback {
         Box::new(move |app: &mut App| {
             app.ui.space_cove_panel.update(&app.world)?;
-            app.ui.switch_to(super::ui::UiTab::SpaceCove);
+            app.ui.switch_to(super::ui_screen::UiTab::SpaceCove);
 
             Ok(None)
         })
@@ -370,7 +396,7 @@ impl UiCallback {
             app.ui
                 .galaxy_panel
                 .go_to_planet(team.home_planet_id, ZoomLevel::In);
-            app.ui.switch_to(super::ui::UiTab::Galaxy);
+            app.ui.switch_to(super::ui_screen::UiTab::Galaxy);
 
             Ok(None)
         })
@@ -395,7 +421,7 @@ impl UiCallback {
             };
 
             app.ui.galaxy_panel.go_to_planet(target.id, ZoomLevel::In);
-            app.ui.switch_to(super::ui::UiTab::Galaxy);
+            app.ui.switch_to(super::ui_screen::UiTab::Galaxy);
 
             Ok(None)
         })
@@ -411,7 +437,7 @@ impl UiCallback {
                 } => {
                     let target = app.world.get_planet_or_err(&current_planet_id)?;
                     app.ui.galaxy_panel.go_to_planet(target.id, ZoomLevel::In);
-                    app.ui.switch_to(super::ui::UiTab::Galaxy);
+                    app.ui.switch_to(super::ui_screen::UiTab::Galaxy);
                 }
                 PlayerLocation::WithTeam => {
                     return Self::go_to_current_team_planet(player.team.unwrap())(app);
@@ -425,7 +451,7 @@ impl UiCallback {
     fn go_to_planet_zoom_in(planet_id: PlanetId) -> AppCallback {
         Box::new(move |app: &mut App| {
             app.ui.galaxy_panel.go_to_planet(planet_id, ZoomLevel::In);
-            app.ui.switch_to(super::ui::UiTab::Galaxy);
+            app.ui.switch_to(super::ui_screen::UiTab::Galaxy);
             Ok(None)
         })
     }
@@ -433,7 +459,7 @@ impl UiCallback {
     fn go_to_planet_zoom_out(planet_id: PlanetId) -> AppCallback {
         Box::new(move |app: &mut App| {
             app.ui.galaxy_panel.go_to_planet(planet_id, ZoomLevel::Out);
-            app.ui.switch_to(super::ui::UiTab::Galaxy);
+            app.ui.switch_to(super::ui_screen::UiTab::Galaxy);
             Ok(None)
         })
     }
@@ -516,7 +542,7 @@ impl UiCallback {
 
             app.ui.game_panel.update(&app.world)?;
             app.ui.game_panel.set_active_game(game_id)?;
-            app.ui.switch_to(super::ui::UiTab::Games);
+            app.ui.switch_to(super::ui_screen::UiTab::Games);
             Ok(Some("Challenge accepted".to_string()))
         })
     }
@@ -549,7 +575,7 @@ impl UiCallback {
             }
 
             // Local trade
-            if proposer_player.bare_value() >= target_player.bare_value() {
+            if proposer_player.bare_hiring_value() >= target_player.bare_hiring_value() {
                 app.world
                     .swap_players_team(proposer_player_id, target_player_id)?;
 
@@ -630,6 +656,72 @@ impl UiCallback {
         })
     }
 
+    fn organize_new_tournament(max_participants: usize, starting_at: Tick) -> AppCallback {
+        Box::new(move |app: &mut App| {
+            let own_team = app
+                .world
+                .teams
+                .get_mut(&app.world.own_team_id)
+                .expect("Own team should exists");
+            own_team.can_organize_tournament()?;
+
+            let planet_id = own_team
+                .has_space_cove_on()
+                .expect("Team should have a space cove");
+            let planet = app
+                .world
+                .planets
+                .get(&planet_id)
+                .expect("Space cove planet should exist.");
+
+            let mut tournament =
+                Tournament::new(own_team, max_participants, starting_at)?.on_planet(planet);
+
+            let mut players = PlayerMap::new();
+            for &player_id in own_team.player_ids.iter().take(MAX_PLAYERS_PER_GAME) {
+                let player = if let Some(player) = app.world.players.get(&player_id) {
+                    player.clone()
+                } else {
+                    return Err(anyhow!("Could not find player {player_id}"));
+                };
+                players.insert(player_id, player);
+            }
+
+            tournament.register_team(own_team, players)?;
+            app.world
+                .tournaments
+                .insert(tournament.id, tournament.clone());
+
+            own_team.tournament_registration_state = TournamentRegistrationState::Confirmed {
+                tournament_id: tournament.id,
+            };
+
+            app.ui.push_popup_to_top(PopupMessage::Ok {
+                message: format!(
+                    "New tournament created! \nStarting on {} {} on {} \n {}/{} registered teams",
+                    starting_at.formatted_as_date(),
+                    starting_at.formatted_as_time(),
+                    planet.name,
+                    tournament.participants.len(),
+                    tournament.max_participants
+                ),
+                is_skippable: true,
+                tick: Tick::now(),
+            });
+
+            if let Err(err) = app.network_handler.send_tournament(tournament) {
+                app.ui.push_log_event(
+                    Tick::now(),
+                    None,
+                    format!("Cannot send tournament: {err}"),
+                    log::Level::Error,
+                );
+            }
+
+            Ok(None)
+        })
+    }
+
     fn cancel_generate_own_team() -> AppCallback {
         Box::new(move |app: &mut App| {
             app.ui.new_team_screen.set_state(CreationState::Players);
@@ -673,9 +765,7 @@ impl UiCallback {
     fn next_training_focus(team_id: TeamId) -> AppCallback {
         Box::new(move |app: &mut App| {
             let mut team = app.world.get_team_or_err(&team_id)?.clone();
-            if team.current_game.is_some() {
-                return Err(anyhow!("Cannot change training focus:\nTeam is playing"));
-            }
+            team.can_change_training_focus()?;
 
             let new_focus = match team.training_focus {
                 Some(focus) => focus.next(),
@@ -1075,6 +1165,7 @@ impl UiCallback {
             Self::GoToTeam { team_id } => Self::go_to_team(*team_id)(app),
             Self::GoToPlayer { player_id } => Self::go_to_player(*player_id)(app),
             Self::GoToPlayerTeam { player_id } => Self::go_to_player_team(*player_id)(app),
+            Self::GoToLoadedGame { game } => Self::go_to_loaded_game(game.clone())(app),
             Self::GoToGame { game_id } => Self::go_to_game(*game_id)(app),
             Self::GoToPlanet { planet_id } => Self::go_to_planet(*planet_id)(app),
             Self::GoToSpaceCove => Self::go_to_space_cove()(app),
@@ -1198,7 +1289,7 @@ impl UiCallback {
             }
             Self::ContinueGame => {
                 app.ui.splash_screen.set_index(0);
-                app.load_world();
+                app.continue_game();
                 Ok(None)
             }
             Self::QuitGame => {
@@ -1249,6 +1340,10 @@ impl UiCallback {
                 app.ui.team_panel.set_view(*view);
                 Ok(None)
             }
+            Self::SetTournamentPanelView { view } => {
+                app.ui.tournament_panel.set_view(*view);
+                Ok(None)
+            }
             Self::AbandonAsteroid { asteroid_id } => {
                 let own_team = app.world.get_own_team_mut()?;
                 own_team.asteroid_ids.retain(|&id| id != *asteroid_id);
@@ -1326,7 +1421,7 @@ impl UiCallback {
                 let rng = &mut ChaCha8Rng::from_os_rng();
 
                 let discovery_probability = (PORTAL_DISCOVERY_PROBABILITY
-                    * TeamBonus::Exploration.current_player_bonus(&player)? as f64)
+                    * TeamBonus::Exploration.current_player_bonus(&player) as f64)
                     .min(1.0);
                 if matches!(player.special_trait, Some(Trait::Spugna))
                     && player.info.crew_role == CrewRole::Pilot
@@ -1412,6 +1507,10 @@ impl UiCallback {
                 players.clone(),
                 spaceship.clone(),
             )(app),
+            Self::OrganizeNewTournament {
+                max_participants,
+                starting_at,
+            } => Self::organize_new_tournament(*max_participants, *starting_at)(app),
             Self::CancelGeneratePlayerTeam => Self::cancel_generate_own_team()(app),
             Self::AssignBestTeamPositions => Self::assign_best_team_positions()(app),
             Self::SwapPlayerPositions {
@@ -1577,8 +1676,6 @@ impl UiCallback {
                 }
 
                 if let Some(asteroid_type) = space.asteroid_planet_found() {
-                    own_team.reputation =
-                        (own_team.reputation + ReputationModifier::SMALL_BONUS).bound();
                     app.ui.push_popup(PopupMessage::AsteroidNameDialog {
                         tick: Tick::now(),
                         asteroid_type,
@@ -1719,10 +1816,10 @@ mod test {
     use super::UiCallback;
     use crate::{
         app::App,
+        core::resources::Resource,
         space_adventure::{ControllableSpaceship, GameEntity, SpaceCallback},
         types::{AppResult, ResourceMap, StorableResourceMap},
         ui::ui_callback::INITIAL_TEAM_BALANCE,
-        world::resources::Resource,
     };
 
     #[test]
