@@ -1,15 +1,13 @@
 #[cfg(test)]
 mod tests {
     use itertools::Itertools;
-    use rand::SeedableRng;
-    use rand_chacha::ChaCha8Rng;
     use rayon::prelude::*;
+    use rebels::core::{Player, Rated, Skill, Team, TickInterval, MAX_PLAYERS_PER_GAME};
+    use rebels::game_engine::action::{ActionOutput, ActionSituation, Advantage};
     use rebels::game_engine::game::Game;
     use rebels::game_engine::tactic::Tactic;
     use rebels::game_engine::types::{GameStats, GameStatsMap, Possession, TeamInGame};
-    use rebels::types::{AppResult, GameId, PlayerId, PlayerMap, TeamId};
-    use rebels::types::{SystemTimeTick, Tick};
-    use rebels::world::{Planet, Player, Rated, Team, TickInterval, MAX_PLAYERS_PER_GAME};
+    use rebels::types::{AppResult, PlayerId, PlayerMap, SystemTimeTick, TeamId, Tick};
     use std::collections::{BTreeMap, HashMap};
     use strum::IntoEnumIterator;
 
@@ -40,6 +38,7 @@ mod tests {
         away_stats: GameStatsMap,
         home_players: PlayerMap,
         away_players: PlayerMap,
+        action_outputs: Vec<ActionOutput>,
     }
 
     fn process_stats(samples: &[GameSample], bin_size: f32) -> Vec<BinResult> {
@@ -81,6 +80,7 @@ mod tests {
             |s: &GameStats, _: &Player| s.blocks as f32,
             |s: &GameStats, _: &Player| s.brawls[0] as f32 + 0.5 * s.brawls[1] as f32,
             |_: &GameStats, p: &Player| p.tiredness,
+            |_: &GameStats, p: &Player| p.morale,
         ];
 
         let binned = compute_binned_stats(samples, bin_size, selectors);
@@ -123,7 +123,10 @@ mod tests {
             .sum()
     }
 
-    fn generate_team_in_game(rng: &mut ChaCha8Rng, team_base_level: f32) -> TeamInGame {
+    fn generate_team_in_game(
+        team_base_level: f32,
+        with_fixed_stamina: Option<Skill>,
+    ) -> TeamInGame {
         let team = Team {
             id: TeamId::new_v4(),
             ..Default::default()
@@ -131,7 +134,12 @@ mod tests {
 
         let mut players = PlayerMap::new();
         for _ in 0..MAX_PLAYERS_PER_GAME {
-            let player = Player::random(rng, None, &Planet::default(), team_base_level);
+            let mut player = Player::default()
+                .with_base_level(team_base_level)
+                .randomize(None);
+            if let Some(value) = with_fixed_stamina {
+                player.athletics.stamina = value;
+            }
             players.insert(player.id, player);
         }
 
@@ -139,8 +147,8 @@ mod tests {
     }
 
     fn generate_identical_team_in_game(
-        rng: &mut ChaCha8Rng,
         team_base_level: f32,
+        with_fixed_stamina: Option<Skill>,
     ) -> (TeamInGame, TeamInGame) {
         let home_team = Team {
             id: TeamId::new_v4(),
@@ -153,10 +161,18 @@ mod tests {
         let mut home_players = PlayerMap::new();
         let mut away_players = PlayerMap::new();
         for _ in 0..MAX_PLAYERS_PER_GAME {
-            let player = Player::random(rng, None, &Planet::default(), team_base_level);
-            let mut away_player = player.clone();
+            let mut home_player = Player::default()
+                .with_base_level(team_base_level)
+                .randomize(None);
+            let mut away_player = home_player.clone();
+
+            if let Some(value) = with_fixed_stamina {
+                home_player.athletics.stamina = value;
+                away_player.athletics.stamina = value;
+            }
+
             away_player.id = PlayerId::new_v4();
-            home_players.insert(player.id, player);
+            home_players.insert(home_player.id, home_player);
             away_players.insert(away_player.id, away_player);
         }
 
@@ -171,31 +187,32 @@ mod tests {
         max_delta_rating: f32,
         home_tactic: Tactic,
         away_tactic: Tactic,
+        with_fixed_stamina: Option<Skill>,
     ) -> Vec<GameSample> {
-        let rng = &mut ChaCha8Rng::from_os_rng();
         let mut samples = Vec::with_capacity(n_games);
         for i in 0..n_games {
-            let home_team_base_level = if (i as f32) <= (n_games as f32) / (2.0 * max_delta_rating)
-            {
-                0.0
-            } else {
-                max_delta_rating * i as f32 / n_games as f32
-            };
-            let away_team_base_level = if (i as f32) <= (n_games as f32) / (2.0 * max_delta_rating)
-            {
-                0.0
-            } else {
-                -max_delta_rating * i as f32 / n_games as f32
-            };
+            let home_team_base_level =
+                if max_delta_rating > 0.0 && i <= n_games / (2 * max_delta_rating as usize) {
+                    0.0
+                } else {
+                    max_delta_rating * i as f32 / n_games as f32
+                };
+            let away_team_base_level =
+                if max_delta_rating > 0.0 && i <= n_games / (2 * max_delta_rating as usize) {
+                    0.0
+                } else {
+                    -max_delta_rating * i as f32 / n_games as f32
+                };
 
-            let (home_team_in_game, away_team_in_game) = if max_delta_rating == 0.0 {
-                generate_identical_team_in_game(rng, home_team_base_level)
-            } else {
-                (
-                    generate_team_in_game(rng, home_team_base_level),
-                    generate_team_in_game(rng, away_team_base_level),
-                )
-            };
+            let (home_team_in_game, away_team_in_game) =
+                if home_team_base_level == away_team_base_level {
+                    generate_identical_team_in_game(home_team_base_level, with_fixed_stamina)
+                } else {
+                    (
+                        generate_team_in_game(home_team_base_level, with_fixed_stamina),
+                        generate_team_in_game(away_team_base_level, with_fixed_stamina),
+                    )
+                };
 
             let home_rating = home_team_in_game.rating();
             let away_rating = away_team_in_game.rating();
@@ -220,14 +237,7 @@ mod tests {
             away_team_in_game.tactic = away_tactic;
 
             let mut current_tick = Tick::now();
-
-            let mut game = Game::new(
-                GameId::new_v4(),
-                home_team_in_game,
-                away_team_in_game,
-                current_tick,
-                &Planet::default(),
-            );
+            let mut game = Game::test(home_team_in_game, away_team_in_game);
 
             // Simulate until finished
             while !game.has_ended() {
@@ -244,6 +254,8 @@ mod tests {
                 _ => unreachable!(),
             };
 
+            let action_outputs = game.action_results;
+
             samples.push(GameSample {
                 rating_diff,
                 winner,
@@ -251,25 +263,28 @@ mod tests {
                 away_stats: game.away_team_in_game.stats,
                 home_players: game.home_team_in_game.players,
                 away_players: game.away_team_in_game.players,
+                action_outputs,
             })
         }
 
         samples
     }
 
-    /// Returns (mean, stddev, count) per bin_center (int)
     fn compute_binned_stats<F>(
         samples: &[GameSample], // (rating_diff, home_stats, away_stats)
         bin_size: f32,
         selectors: Vec<F>,
     ) -> BTreeMap<i32, ((Vec<f32>, Vec<f32>), (Vec<f32>, Vec<f32>), usize)>
+    // Returns bin_center --> ((home mean, home stddev) for each selector, (away mean, away stddev) for each selector, count)
     where
         F: Fn(&GameStats, &Player) -> f32,
     {
+        let entry_length = selectors.len() + 3 + 1 + 1; // +3 for advantages + 1 for fastbreak +1 for substitution
+
         // First pass: sum and count for each selector
         let default_entry = (
-            vec![0.0f32].repeat(selectors.len()), // away avg/stddev for each selector
-            vec![0.0f32].repeat(selectors.len()), // home avg/stddev for each selector
+            vec![0.0f32].repeat(entry_length),
+            vec![0.0f32].repeat(entry_length),
             0usize,
         );
         let mut sums: BTreeMap<i32, (Vec<f32>, Vec<f32>, usize)> = BTreeMap::new();
@@ -280,17 +295,84 @@ mod tests {
                 entry.0[idx] += team_stat_sum(&sample.home_stats, &sample.home_players, selector);
                 entry.1[idx] += team_stat_sum(&sample.away_stats, &sample.away_players, selector);
             }
+
+            // Push data from sample.action_outputs
+            for &possession in [Possession::Home, Possession::Away].iter() {
+                for (idx, &advantage) in [Advantage::Attack, Advantage::Neutral, Advantage::Defense]
+                    .iter()
+                    .enumerate()
+                {
+                    let advantage_count = sample
+                        .action_outputs
+                        .iter()
+                        .filter(|output| {
+                            output.possession == possession && output.advantage == advantage
+                        })
+                        .count() as f32;
+                    if possession == Possession::Home {
+                        entry.0[selectors.len() + idx] += advantage_count;
+                    } else {
+                        entry.1[selectors.len() + idx] += advantage_count;
+                    }
+                }
+
+                let fastbreak_count = sample
+                    .action_outputs
+                    .iter()
+                    .filter(|output| {
+                        output.possession == possession
+                            && output.situation == ActionSituation::Fastbreak
+                    })
+                    .count() as f32;
+                if possession == Possession::Home {
+                    entry.0[selectors.len() + 3] += fastbreak_count;
+                } else {
+                    entry.1[selectors.len() + 3] += fastbreak_count;
+                }
+
+                let home_sub_count = sample
+                    .action_outputs
+                    .iter()
+                    .filter(|output| {
+                        output.situation == ActionSituation::AfterSubstitution
+                            && ((output.attack_stats_update.is_some()
+                                && output.possession == Possession::Home)
+                                || (output.defense_stats_update.is_some()
+                                    && output.possession == Possession::Away))
+                    })
+                    .count() as f32;
+
+                let away_sub_count = sample
+                    .action_outputs
+                    .iter()
+                    .filter(|output| {
+                        output.situation == ActionSituation::AfterSubstitution
+                            && ((output.attack_stats_update.is_some()
+                                && output.possession == Possession::Away)
+                                || (output.defense_stats_update.is_some()
+                                    && output.possession == Possession::Home))
+                    })
+                    .count() as f32;
+
+                if possession == Possession::Home {
+                    entry.0[selectors.len() + 4] += home_sub_count;
+                } else {
+                    entry.1[selectors.len() + 4] += away_sub_count;
+                }
+            }
+
+            // Increase count
             entry.2 += 1;
         }
 
         // Means
         let mut means: BTreeMap<i32, (Vec<f32>, Vec<f32>)> = BTreeMap::new();
         for (bin, (home_sums, away_sums, count)) in &sums {
-            let home_means = (0..selectors.len())
+            let home_means = (0..entry_length)
                 .map(|idx| home_sums[idx] / *count as f32)
                 .collect();
 
-            let away_means = (0..selectors.len())
+            let away_means = (0..entry_length)
                 .map(|idx| away_sums[idx] / *count as f32)
                 .collect();
 
@@ -298,15 +380,20 @@ mod tests {
         }
 
         // Second pass: sum squared deviations
+
         let default_entry = (
-            vec![0.0f32].repeat(selectors.len()), // away avg/stddev for each selector
-            vec![0.0f32].repeat(selectors.len()), // home avg/stddev for each selector
+            vec![0.0f32].repeat(entry_length),
+            vec![0.0f32].repeat(entry_length),
         );
+
+        assert!(default_entry.0.len() > selectors.len());
+
         let mut sqdevs: BTreeMap<i32, (Vec<f32>, Vec<f32>)> = BTreeMap::new();
         for sample in samples {
             let bin = (sample.rating_diff / bin_size).round() as i32;
             let (home_means, away_means) = means[&bin].clone();
             let entry = sqdevs.entry(bin).or_insert(default_entry.clone());
+
             for (idx, selector) in selectors.iter().enumerate() {
                 entry.0[idx] += (team_stat_sum(&sample.home_stats, &sample.home_players, selector)
                     - home_means[idx])
@@ -315,6 +402,77 @@ mod tests {
                     - away_means[idx])
                     .powi(2);
             }
+
+            // Push data from sample.action_outputs
+            for &possession in [Possession::Home, Possession::Away].iter() {
+                for (idx, &advantage) in [Advantage::Attack, Advantage::Neutral, Advantage::Defense]
+                    .iter()
+                    .enumerate()
+                {
+                    let advantage_count = sample
+                        .action_outputs
+                        .iter()
+                        .filter(|output| {
+                            output.possession == possession && output.advantage == advantage
+                        })
+                        .count() as f32;
+                    if possession == Possession::Home {
+                        entry.0[selectors.len() + idx] +=
+                            (advantage_count - home_means[selectors.len() + idx]).powi(2);
+                    } else {
+                        entry.1[selectors.len() + idx] +=
+                            (advantage_count - away_means[selectors.len() + idx]).powi(2);
+                    }
+                }
+
+                let fastbreak_count = sample
+                    .action_outputs
+                    .iter()
+                    .filter(|output| {
+                        output.possession == possession
+                            && output.situation == ActionSituation::Fastbreak
+                    })
+                    .count() as f32;
+                if possession == Possession::Home {
+                    entry.0[selectors.len() + 3] +=
+                        (fastbreak_count - home_means[selectors.len() + 3]).powi(2);
+                } else {
+                    entry.1[selectors.len() + 3] +=
+                        (fastbreak_count - away_means[selectors.len() + 3]).powi(2);
+                }
+
+                let home_sub_count = sample
+                    .action_outputs
+                    .iter()
+                    .filter(|output| {
+                        output.situation == ActionSituation::AfterSubstitution
+                            && ((output.attack_stats_update.is_some()
+                                && output.possession == Possession::Home)
+                                || (output.defense_stats_update.is_some()
+                                    && output.possession == Possession::Away))
+                    })
+                    .count() as f32;
+
+                let away_sub_count = sample
+                    .action_outputs
+                    .iter()
+                    .filter(|output| {
+                        output.situation == ActionSituation::AfterSubstitution
+                            && ((output.attack_stats_update.is_some()
+                                && output.possession == Possession::Away)
+                                || (output.defense_stats_update.is_some()
+                                    && output.possession == Possession::Home))
+                    })
+                    .count() as f32;
+
+                if possession == Possession::Home {
+                    entry.0[selectors.len() + 4] +=
+                        (home_sub_count - home_means[selectors.len() + 4]).powi(2);
+                } else {
+                    entry.1[selectors.len() + 4] +=
+                        (away_sub_count - away_means[selectors.len() + 4]).powi(2);
+                }
+            }
         }
 
         // Final assembly: compute sample variance (N-1), stddev, and count
@@ -467,6 +625,16 @@ mod tests {
                 bin.home_avg[11], bin.home_std[11], bin.away_avg[11], bin.away_std[11],
             );
 
+            println!(
+                "  Fastbreaks = {:3.1} ± {:3.1} vs {:3.1} ± {:3.1}",
+                bin.home_avg[17], bin.home_std[17], bin.away_avg[17], bin.away_std[17],
+            );
+
+            println!(
+                "  Substitutions = {:3.1} ± {:3.1} vs {:3.1} ± {:3.1}",
+                bin.home_avg[18], bin.home_std[18], bin.away_avg[18], bin.away_std[18],
+            );
+
             println!(
                 "  Tiredness = {:3.1} ± {:3.1} vs {:3.1} ± {:3.1}",
                 bin.home_avg[12] / MAX_PLAYERS_PER_GAME as f32,
@@ -474,6 +642,23 @@ mod tests {
                 bin.away_avg[12] / MAX_PLAYERS_PER_GAME as f32,
                 bin.away_std[12] / MAX_PLAYERS_PER_GAME as f32,
             );
+            println!(
+                "  Morale = {:3.1} ± {:3.1} vs {:3.1} ± {:3.1}",
+                bin.home_avg[13] / MAX_PLAYERS_PER_GAME as f32,
+                bin.home_std[13] / MAX_PLAYERS_PER_GAME as f32,
+                bin.away_avg[13] / MAX_PLAYERS_PER_GAME as f32,
+                bin.away_std[13] / MAX_PLAYERS_PER_GAME as f32,
+            );
+
+            println!(
+                "  Advantage = {:3.1}/{:3.1}/{:3.1} vs {:3.1}/{:3.1}/{:3.1}",
+                bin.home_avg[14],
+                bin.home_avg[15],
+                bin.home_avg[16],
+                bin.away_avg[14],
+                bin.away_avg[15],
+                bin.away_avg[16],
+            );
 
             println!("");
         }
@@ -482,9 +667,10 @@ mod tests {
     #[ignore]
     #[test]
     fn test_multiple_games() -> AppResult<()> {
-        const N: usize = 100_000;
+        const N: usize = 6_000;
         const BIN_SIZE: f32 = 1.0;
-        let max_delta_rating: f32 = 0.0;
+        let max_delta_rating: f32 = 3.0;
+        let with_fixed_stamina = None; //Some(10.0);
 
         let tactic_pairs = Tactic::iter()
             .enumerate()
@@ -499,8 +685,13 @@ mod tests {
         let results: Vec<MatchupResult> = tactic_pairs
             .par_iter()
             .map(|&(home_tactic, away_tactic)| {
-                let samples =
-                    get_simulated_game_samples(N, max_delta_rating, home_tactic, away_tactic);
+                let samples = get_simulated_game_samples(
+                    N,
+                    max_delta_rating,
+                    home_tactic,
+                    away_tactic,
+                    with_fixed_stamina,
+                );
                 let bins = process_stats(&samples, BIN_SIZE);
                 MatchupResult {
                     home_tactic,
@@ -519,4 +710,4 @@ mod tests {
     }
 }
 
-//cargo test test_multiple_games -- --nocapture --ignored > tests/game_stats.data
+//cargo test test_multiple_games -- --nocapture --ignored > tests/game_engine_data/game_stats_v1.5.x.data
